@@ -4,25 +4,24 @@ import ssl
 import time
 import shutil
 import urllib
-import random
-import string
 import logging
-import requests
 import posixpath
 import traceback
 import threading
 import contextlib
 
-from requests.exceptions import InvalidSchema, InvalidURL, MissingSchema
 from pathlib import Path
 from http.cookiejar import MozillaCookieJar
 from urllib.error import ContentTooShortError
 import urllib.parse as urlparse
 from email.utils import unquote
 
+import requests
 import html2text
 import youtube_dl
-from youtube_dl.utils import format_bytes
+
+from youtube_dl.utils import format_bytes, timeconvert
+from requests.exceptions import InvalidSchema, InvalidURL, MissingSchema
 
 from moodle_dl.state_recorder.file import File
 from moodle_dl.state_recorder.course import Course
@@ -292,46 +291,35 @@ class URLTarget(object):
             self.thread_report[self.thread_id]['percentage'] = 100
             self.thread_report[self.thread_id]['extra_totalsize'] = None
 
-    def move_tmp_file(self, tmp_file: str):
+    def is_blocked_for_youtube_dl(self, url_to_download: str):
+        url_parsed = urlparse.urlparse(url_to_download)
+        if url_parsed.hostname.endswith('youtube.com') and url_parsed.path.startswith('/channel/'):
+            return True
+        return False
+
+    def set_utime(self, last_modified: str = None):
+        """Sets the last modified and last activated time of a downloaded file
+
+        Args:
+            last_modified (str, optional): The last_modified header from the Webpage. Defaults to None.
         """
-        Moves temporary files to there correct locations.
-        This tries to move every file that beginns with the tmp_file string
-        to its new locations.
-        @params tmp_file: Is a path + the basename
-                          (without the extension) of the tmp_file
-        """
-        lookup_dir = os.path.dirname(tmp_file)
-        tmp_filename = os.path.basename(tmp_file)
 
-        logging.debug('T%s - Renaming "%s" to "%s"', self.thread_id, tmp_filename, self.filename)
-        for found_filename in os.listdir(lookup_dir):
-            if found_filename.startswith(tmp_filename + '.'):
-                one_tmp_file = os.path.join(lookup_dir, found_filename)
-
-                found_filename_base, found_file_extension = os.path.splitext(found_filename)
-
-                new_filename = self.filename + found_file_extension
-                new_path = str(Path(self.destination) / new_filename)
-
-                # lock because of raise condition
-                self.fs_lock.acquire()
-                new_path = self._get_path_of_non_existent_file(new_path)
-
-                self.file.saved_to = new_path
+        if last_modified is not None:
+            filetime = timeconvert(last_modified)
+            if filetime is not None and filetime >= 0:
                 try:
-                    logging.debug('T%s - Moved "%s" to "%s"', self.thread_id, found_filename, new_filename)
-                    shutil.move(one_tmp_file, self.file.saved_to)
-                except Exception as e:
-                    logging.warning(
-                        'T%s - Failed to move the temporary video file %s to %s!  Error: %s',
-                        self.thread_id,
-                        found_filename,
-                        new_filename,
-                        e,
-                    )
-                self.fs_lock.release()
+                    os.utime(self.file.saved_to, (time.time(), filetime))
+                    return
+                except Exception:
+                    logging.debug('T%s - Could not change utime', self.thread_id)
+                    pass
 
-        self.file.time_stamp = int(time.time())
+        try:
+            os.utime(self.file.saved_to, (time.time(), self.file.content_timemodified))
+            return
+        except Exception:
+            logging.debug('T%s - Could not change utime', self.thread_id)
+            pass
 
     def try_download_link(
         self, add_token: bool = False, delete_if_successful: bool = False, use_cookies: bool = False
@@ -350,11 +338,11 @@ class URLTarget(object):
             bool: If it was successfull.
         """
 
-        urlToDownload = self.file.content_fileurl
-        logging.debug('T%s - Try to download linked file %s', self.thread_id, urlToDownload)
+        url_to_download = self.file.content_fileurl
+        logging.debug('T%s - Try to download linked file %s', self.thread_id, url_to_download)
 
         if add_token:
-            urlToDownload = self._add_token_to_url(self.file.content_fileurl)
+            url_to_download = self._add_token_to_url(self.file.content_fileurl)
 
         cookies_path = None
         if use_cookies:
@@ -390,8 +378,8 @@ class URLTarget(object):
 
         try:
             response = session.head(
-                urlToDownload,
-                headers=RequestHelper.desktopHeader,
+                url_to_download,
+                headers=RequestHelper.stdHeader,
                 verify=self.verify_cert,
                 allow_redirects=True,
             )
@@ -417,8 +405,16 @@ class URLTarget(object):
             isHTML = True
 
         total_bytes_estimate = int(response.headers.get('Content-Length', -1))
+        last_modified = response.headers.get('Last-Modified', None)
 
-        url_parsed = urlparse.urlsplit(urlToDownload)
+        if response.url != url_to_download:
+            if response.history and len(response.history) > 0:
+                logging.debug('T%s - URL was %s time(s) redirected', self.thread_id, len(response.history))
+            else:
+                logging.debug('T%s - URL has changed after information retrieval', self.thread_id)
+            url_to_download = response.url
+
+        url_parsed = urlparse.urlparse(url_to_download)
         new_filename = posixpath.basename(url_parsed.path)
 
         if "Content-Disposition" in response.headers.keys():
@@ -426,28 +422,39 @@ class URLTarget(object):
             if len(found_names) > 0:
                 new_filename = unquote(found_names[0])
 
-        if isHTML:
-            tmp_filename = ''.join(random.choices(string.ascii_uppercase + string.digits, k=10))
-            tmp_file = str(Path(self.destination) / tmp_filename)
+        if isHTML and not self.is_blocked_for_youtube_dl(url_to_download):
+
+            filename_tmpl = self.filename + ' | %(title)s (%(id)s).%(ext)s'
+            if self.file.content_type == 'description-url':
+                filename_tmpl = '%(title)s (%(id)s).%(ext)s'
+            outtmpl = str(Path(self.destination) / filename_tmpl)
+
             ydl_opts = {
                 'logger': self.YtLogger(self),
                 'progress_hooks': [self.yt_hook],
-                'outtmpl': (tmp_file + '.%(ext)s'),
-                'nocheckcertificate': True,
+                'outtmpl': outtmpl,
+                'nocheckcertificate': self.skip_cert_verify,
                 'retries': 10,
                 'fragment_retries': 10,
+                'ignoreerrors': True,
+                'addmetadata': True,
             }
+            youtube_dl_options = self.options.get('youtube_dl_options', {})
+            ydl_opts.update(youtube_dl_options)
+
             if use_cookies:
                 ydl_opts.update({'cookiefile': cookies_path})
 
             with youtube_dl.YoutubeDL(ydl_opts) as ydl:
                 try:
-                    ydl_results = ydl.download([urlToDownload])
+                    ydl_results = ydl.download([url_to_download])
                     if ydl_results == 1:
                         # return False
                         pass
-                    else:
-                        self.move_tmp_file(tmp_file)
+                    elif self.file.module_name != 'index_mod-page':
+                        self.file.saved_to = str(Path(self.destination) / self.filename)
+                        self.file.time_stamp = int(time.time())
+
                         self.success = True
                         return True
                 except Exception:
@@ -467,14 +474,9 @@ class URLTarget(object):
                             e,
                         )
                 self.success = False
-                raise RuntimeError('Youtube-dl failed to download the URL, see youtube-dl error message for details.')
-
-        if response.url != urlToDownload:
-            if response.history and len(response.history) > 0:
-                logging.debug('T%s - URL was redirected, %s time(s)!', self.thread_id, len(response.history))
-            else:
-                logging.debug('T%s - URL has changed after information retrieval.', self.thread_id)
-            urlToDownload = response.url
+                raise RuntimeError(
+                    'Youtube-dl could not download the URL. For details see youtube-dl error messages in the log file'
+                )
 
         logging.debug('T%s - Downloading file directly', self.thread_id)
 
@@ -483,8 +485,8 @@ class URLTarget(object):
         if new_extension == '' and isHTML:
             new_extension = '.html'
 
-        # if self.filename.startswith(('https://', 'http://')):
-        #     self.filename = new_name + new_extension
+        if self.file.content_type == 'description-url' and new_name != '':
+            self.filename = new_name + new_extension
 
         old_name, old_extension = os.path.splitext(self.filename)
 
@@ -497,13 +499,14 @@ class URLTarget(object):
             self.thread_report[self.thread_id]['extra_totalsize'] = total_bytes_estimate
 
         self.urlretrieve(
-            urlToDownload,
+            url_to_download,
             self.file.saved_to,
             context=self.ssl_context,
             reporthook=self.add_progress,
             cookies_path=cookies_path,
         )
 
+        self.set_utime(last_modified)
         self.file.time_stamp = int(time.time())
 
         self.success = True
@@ -519,26 +522,26 @@ class URLTarget(object):
         """
 
         url_parsed = urlparse.urlparse(self.file.content_fileurl)
-        domain = url_parsed.netloc.split(':')[0]
+        domain = url_parsed.hostname
 
         blacklist = self.options.get('download_domains_blacklist', [])
         whitelist = self.options.get('download_domains_whitelist', [])
 
-        inBlacklist = False
+        in_blacklist = False
 
         for entry in blacklist:
             if domain == entry or domain.endswith('.' + entry):
-                inBlacklist = True
+                in_blacklist = True
                 break
 
-        inWhitelist = len(whitelist) == 0
+        in_whitelist = len(whitelist) == 0
 
         for entry in whitelist:
             if domain == entry or domain.endswith('.' + entry):
-                inWhitelist = True
+                in_whitelist = True
                 break
 
-        return not inWhitelist or inBlacklist
+        return not in_whitelist or in_blacklist
 
     def create_shortcut(self):
         """
@@ -717,8 +720,6 @@ class URLTarget(object):
                 add_token = False
                 if self.options.get('download_linked_files', False) and not self.is_filtered_external_domain():
                     self.try_download_link(add_token, False, False)
-                    # Warning: try_download_link overwrites saved_to and
-                    # time_stamp in move_tmp_file
                 return self.success
 
             url_to_download = self.file.content_fileurl
@@ -735,6 +736,7 @@ class URLTarget(object):
                 cookies_path=cookies_path,
             )
 
+            self.set_utime()
             self.file.time_stamp = int(time.time())
 
             self.success = True
